// File:           bllibi.h - inline method declarations for BLlib
//                 -*- C++ -*-
//
// Package:        Library
//
// Author:         Dr. Simone Zanella
//
// E-Mail:         szp@zanella-hifi.com
//
// Web Address:    http://www.zanella-hifi.com/szp
//
// Copyright (C) 1997-2002 Simone Zanella Productions
//
// This program is free software; you can redistribute
// it and/or modify it under the terms of GNU General Public License
// as published by the Free Software Foundation; either version 2 of
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implicit warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// History:
//
// $Log:bllibi.h,v $
// Revision 1.13  97/03/28  12:00:00  12:00:00  szanella (Zanella Simone)
// first stand-alone public release
//

int SignatureToPreset(char); // Forward declaration

inline int
BL_Message::GetItemType(void)
{

  //
  // BL_Message method to retrieve the type of the currently pointed item
  // in the message.
  //
  // Author:       Simone Zanella
  //
  // Version:      1.1
  //
  // Dependencies: SignatureToPreset;
  //
  // Parameters:   none;
  //
  // Returns:      type value, or BL_NOT_VALID if item pointer is beyond
  //               the last item or item type is unknown
  //


  if (ulBufOffset == ulDataLen)
    return BL_NOT_VALID; // Pointer at the end of buffer

  return SignatureToPreset(*(cpMsgBuf+ulBufOffset));

}


inline
BL_Message::~BL_Message()
{
  // Destructor: frees up memory allocated to cpMsgBuf
  free(cpMsgBuf);
}


inline
BL_Message::BL_Message()
{
  // Constructor: initialize attributes

  i32Type = BL_NOT_DEFINED;
  ulDataLen = 0;
  cpMsgBuf = NULL;
  ulBufOffset = 0;
  ulCurrentItem = 1;
}


inline void
BL_Message::SetType (BL_INT32 i32UserType)
{
  // Method used by the user to set the type tag

  i32Type = i32UserType;
}


inline BL_INT32
BL_Message::GetType ()
{
  // Method to retrieve the type tag

  return i32Type;
}


inline unsigned long
BL_Message::GetDataLen ()
{
  // Method to retrieve total message buffer length

  return ulDataLen;
}


inline int
BL_Message::EOMsg ()
{
  // Method to determine if item pointer is at end of message

  return (ulBufOffset == ulDataLen);
}

//******************************************************************
//************************** END OF FILE ***************************
//******************************************************************
