// File:           bltest.cpp - -*- C++ -*-
//
// Description:    tests for BLlib
//
// Package:        Library
//
// Author:         Dr. Simone Zanella
//
// E-Mail:         szp@zanella-hifi.com
//
// Web Address:    http://www.zanella-hifi.com/szp
//
// Copyright (C) 1997-2002 Simone Zanella Productions
//
// This program is free software; you can redistribute
// it and/or modify it under the terms of GNU General Public License
// as published by the Free Software Foundation; either version 2 of
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implicit warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// History:
//
// $Log:bltest.cpp,v $
// Revision 1.13  97/03/28  12:00:00  12:00:00  szanella (Zanella Simone)
// first stand-alone public release
//


#include "bllib.h"
#include <limits.h>
#include <float.h>
#include <stdio.h>

#ifndef BL_MSD_INCLUDE
 #include <streambuf.h>
 #include <stream.h>
#endif

#define MAX_STRING 255
#define RW_DATA_SIZE 35000ul

void OutOk();
void OutFail();

int
main()
{

  BL_INT32 i32UserType;

  char cValue;
  unsigned char ucValue;
  short sValue, sGap;
  unsigned short usValue;
  int iValue;
  unsigned int uiValue;
  long lValue;
  unsigned long ulValue;
  float fValue;
  double dValue;
  BL_Charp cpValue;
  BL_RawData rwValue;

  short sTestType, sChoice;

  BL_Message mymsg;

  cin.width(MAX_STRING);

  cout << "1 - Precision test\n";
  cout << "2 - Send message test\n";
  cout << "3 - Receive message test\n";
  cout << "4 - Send + receive message test\n";
  cout << "5 - Quit\n\n";
  cout << "Make your choice: ";
  cin >> sTestType;

  cout << endl;

  switch (sTestType)
  {
    case 1:

     // The following tests are targeted to the following architectures:
     // - PC 80x86, compiler Borland C++ 3.1;
     // - Linux;
     // - HP-UX.

     // signed tests

     mymsg << (short) (SCHAR_MIN - 1); // (1) CAN'T fit in 8 bit (signed)
     mymsg << (long) (SHRT_MIN - 1l);  // (2) CAN'T fit in 16 bit (signed)
     mymsg << (char) (SCHAR_MIN);      // (3) can fit in 8 bit (signed)
     mymsg << (short) (SHRT_MIN);      // (4) can fit in 16 bit (signed)
     mymsg << (long) (LONG_MIN);       // (5) can fit in 32 bit (signed)

     // unsigned tests

     mymsg << (unsigned short) (UCHAR_MAX + 1);
     // (6) CAN'T fit in 8 bit (unsigned)
     mymsg << (unsigned long) (USHRT_MAX + 1l);
     // (7) CAN'T fit in 16 bit (unsigned)
     mymsg << (unsigned char) (UCHAR_MAX);
     // (8) can fit in 8 bit (unsigned)
     mymsg << (unsigned short) (USHRT_MAX);
     // (9) can fit in 16 bit (unsigned)
     mymsg << (unsigned long) (ULONG_MAX);
     // (10) can fit in 32 bit (unsigned)

     // mixed tests

     mymsg << (short) (SCHAR_MIN);
     // (11) CAN'T fit in 8 bit unsigned (s->u)
     mymsg << (short) (UCHAR_MAX);
     // (12) can fit in 8 bit unsigned (s->u)
     mymsg << (long) (SHRT_MIN);
     // (13) CAN'T fit in 16 bit unsigned (s->u)
     mymsg << (long) (USHRT_MAX);
     // (14) can fit in 16 bit unsigned (s->u)

     mymsg << (unsigned short) (UCHAR_MAX);
     // (15) CAN'T fit in 8 bit signed (u->s)
     mymsg << (unsigned short) (SCHAR_MAX);
     // (16) can fit in 8 bit signed (u->s)
     mymsg << (unsigned long) (USHRT_MAX);
     // (17) CAN'T fit in 16 bit signed (u->s)
     mymsg << (unsigned long) (SHRT_MAX);
     // (18) can fit in 16 bit signed (u->s)

     // fp tests

     mymsg << (((double) FLT_MAX)*2.0);  // (19) CAN'T fit in a float
     mymsg << (double) FLT_MAX;          // (20) can fit in a float
     mymsg << (float) FLT_MAX;           // (21) can fit in a double
     mymsg << (((double) FLT_MIN)/2.0);  // (22) CAN'T fit in a float
     mymsg << (double) FLT_MIN;          // (23) can fit in a float
     mymsg << (float) FLT_MIN;           // (24) can fit in a double

     // rawdata test

     cpValue = (char *) malloc(RW_DATA_SIZE);
     memset(cpValue, 0, RW_DATA_SIZE);
     mymsg << BL_ToRawData(cpValue, RW_DATA_SIZE);
     // (25) should be RW_DATA_SIZE bytes length
     free(cpValue);

     // fp to int test

     mymsg << (float) (USHRT_MAX+1ul); // (26) CAN'T fit in unsigned short
     mymsg << (double) (USHRT_MAX);    // (27) can fit in unsigned short
     mymsg << (float) (SHRT_MIN-1l);   // (28) CAN'T fit in short
     mymsg << (double) (SHRT_MIN);     // (29) can fit in short

     // int to fp test

     mymsg << (unsigned long) USHRT_MAX; // (30) can fit in float & double

     if (!mymsg.MsgSave("test.dat"))
     {
       cout << "Error saving to test.dat";
       exit(1);
     }
     mymsg.Clear();
     if (!mymsg.MsgLoad("test.dat"))
     {
       cout << "Error loading from test.dat";
       exit(1);
     }

     // signed tests

     mymsg >> cValue;
     if (BL_convResult == BL_CONV_OK)
     {
       cout << "(Test 1) Degrading short to char should have failed:";
       OutFail();
     }
     else
     {
       cout << "(Test 1) Could not shrink proper short to char:";
       OutOk();
       cout << "Value expected: " << (short) (SCHAR_MIN - 1) << endl;

       mymsg.SeekItem(1);
       mymsg >> sValue;
       if (BL_convResult != BL_CONV_OK)
         cout << "Warning: failed short extraction as a short.\n";
       else
       {
         cout << "Value extracted as a short: ";
         cout << sValue << endl;
       }

       mymsg.SeekItem(1);
       mymsg >> iValue;
       if (BL_convResult != BL_CONV_OK)
         cout << "Warning: failed short extraction as a int.\n";
       else
       {
         cout << "Value extracted as a int: ";
         cout << iValue << endl;
       }

       mymsg.SeekItem(1);
       mymsg >> lValue;
       if (BL_convResult != BL_CONV_OK)
       {
         cout << "Warning: failed short extraction as a long.\n";
       }
       else
       {
         cout << "Value extracted as a long: ";
         cout << lValue << endl;
       }

     }

     mymsg.SeekItem(2);
     mymsg >> sValue;
     if (BL_convResult == BL_CONV_OK)
     {
       cout << "(Test 2) Degrading long to short should have failed:";
       OutFail();
     }
     else
     {
       cout << "(Test 2) Could not shrink proper long to short:";
       OutOk();
       cout << "Value expected: " << (long) (SHRT_MIN - 1l) << endl;

       mymsg.SeekItem(2);
       mymsg >> lValue;
       if (BL_convResult != BL_CONV_OK)
       {
         cout << "Warning: failed long extraction as a long.\n";
         mymsg.SkipItem(1);
       }
       else
       {
         cout << "Value extracted as a long: ";
         cout << lValue << endl;
       }
     }

     mymsg.SeekItem(3);
     mymsg >> cValue;
     if (BL_convResult != BL_CONV_OK)
     {
       cout << "(Test 3) Char extraction failed:";
       OutFail();
     }
     else
     {
       cout << "(Test 3) Char extraction ok:";
       OutOk();
       cout << "Value expected: " << (char) (SCHAR_MIN) << endl;

       cout << "Value extracted as a char: " << cValue << endl;

       mymsg.SeekItem(3);
       mymsg >> sValue;
       if (BL_convResult != BL_CONV_OK)
         cout << "Warning: failed char extraction as a short.\n";
       else
       {
         cout << "Value extracted as a short: ";
         cout << sValue << endl;
       }

       mymsg.SeekItem(3);
       mymsg >> iValue;
       if (BL_convResult != BL_CONV_OK)
         cout << "Warning: failed char extraction as a int.\n";
       else
       {
         cout << "Value extracted as a int: ";
         cout << iValue << endl;
       }

       mymsg.SeekItem(3);
       mymsg >> lValue;
       if (BL_convResult != BL_CONV_OK)
       {
         cout << "Warning: failed char extraction as a long.\n";
       }
       else
       {
         cout << "Value extracted as a long: ";
         cout << lValue << endl;
       }
     }

     mymsg.SeekItem(4);
     mymsg >> sValue;
     if (BL_convResult != BL_CONV_OK)
     {
       cout << "(Test 4) Short extraction failed:";
       OutFail();
     }
     else
     {
       cout << "(Test 4) Short extraction ok:";
       OutOk();
       cout << "Value expected: " << (short) (SHRT_MIN) << endl;
       cout << "Value extracted as a short: " << sValue << endl;

       mymsg.SeekItem(4);
       mymsg >> iValue;
       if (BL_convResult != BL_CONV_OK)
         cout << "Warning: failed short extraction as a int.\n";
       else
       {
         cout << "Value extracted as a int: ";
         cout << iValue << endl;
       }

       mymsg.SeekItem(4);
       mymsg >> lValue;
       if (BL_convResult != BL_CONV_OK)
       {
         cout << "Warning: failed short extraction as a long.\n";
       }
       else
       {
         cout << "Value extracted as a long: ";
         cout << lValue << endl;
       }
     }

     mymsg.SeekItem(5);
     mymsg >> lValue;
     if (BL_convResult != BL_CONV_OK)
     {
       cout << "(Test 5) Long extraction failed:";
       OutFail();
     }
     else
     {
       cout << "(Test 5) Long extraction ok:";
       OutOk();
       cout << "Value expected: " << (long) (LONG_MIN) << endl;
       cout << "Value extracted as a long: " << lValue << endl;
     }

     // unsigned tests

     mymsg.SeekItem(6);
     mymsg >> ucValue;
     if (BL_convResult == BL_CONV_OK)
     {
       cout << "(Test 6) Degrading ushort to uchar should have failed:";
       OutFail();
     }
     else
     {
       cout << "(Test 6) Could not shrink proper ushort to uchar:";
       OutOk();
       cout << "Value expected: " << (unsigned short) (UCHAR_MAX + 1);
       cout << endl;

       mymsg.SeekItem(6);
       mymsg >> usValue;
       if (BL_convResult != BL_CONV_OK)
         cout << "Warning: failed ushort extraction as a ushort.\n";
       else
       {
         cout << "Value extracted as a ushort: ";
         cout << usValue << endl;
       }

       mymsg.SeekItem(6);
       mymsg >> uiValue;
       if (BL_convResult != BL_CONV_OK)
         cout << "Warning: failed ushort extraction as a uint.\n";
       else
       {
         cout << "Value extracted as a uint: ";
         cout << uiValue << endl;
       }

       mymsg.SeekItem(6);
       mymsg >> ulValue;
       if (BL_convResult != BL_CONV_OK)
       {
         cout << "Warning: failed ushort extraction as a ulong.\n";
       }
       else
       {
         cout << "Value extracted as a ulong: ";
         cout << ulValue << endl;
       }

     }

     mymsg.SeekItem(7);
     mymsg >> usValue;
     if (BL_convResult == BL_CONV_OK)
     {
       cout << "(Test 7) Degrading ulong to ushort should have failed:";
       OutFail();
     }
     else
     {
       cout << "(Test 7) Could not shrink proper ulong to ushort:";
       OutOk();
       cout << "Value expected: " << (unsigned long) (USHRT_MAX + 1l);
       cout << endl;

       mymsg.SeekItem(7);
       mymsg >> ulValue;
       if (BL_convResult != BL_CONV_OK)
       {
         cout << "Warning: failed ulong extraction as a ulong.\n";
       }
       else
       {
         cout << "Value extracted as a ulong: ";
         cout << ulValue << endl;
       }
     }

     mymsg.SeekItem(8);
     mymsg >> ucValue;
     if (BL_convResult != BL_CONV_OK)
     {
       cout << "(Test 8) Uchar extraction failed:";
       OutFail();
     }
     else
     {
       cout << "(Test 8) Uchar extraction ok:";
       OutOk();
       cout << "Value expected: " << (unsigned char) (UCHAR_MAX) << endl;
       cout << "Value extracted as a uchar: " << ucValue << endl;

       mymsg.SeekItem(8);
       mymsg >> usValue;
       if (BL_convResult != BL_CONV_OK)
         cout << "Warning: failed uchar extraction as a ushort.\n";
       else
       {
         cout << "Value extracted as a ushort: ";
         cout << usValue << endl;
       }

       mymsg.SeekItem(8);
       mymsg >> uiValue;
       if (BL_convResult != BL_CONV_OK)
         cout << "Warning: failed uchar extraction as a uint.\n";
       else
       {
         cout << "Value extracted as a uint: ";
         cout << uiValue << endl;
       }

       mymsg.SeekItem(8);
       mymsg >> ulValue;
       if (BL_convResult != BL_CONV_OK)
       {
         cout << "Warning: failed uchar extraction as a ulong.\n";
       }
       else
       {
         cout << "Value extracted as a ulong: ";
         cout << ulValue << endl;
       }
     }

     mymsg.SeekItem(9);
     mymsg >> usValue;
     if (BL_convResult != BL_CONV_OK)
     {
       cout << "(Test 9) Ushort extraction failed:";
       OutFail();
     }
     else
     {
       cout << "(Test 9) Ushort extraction ok:";
       OutOk();
       cout << "Value expected: " << (unsigned short) (USHRT_MAX) << endl;
       cout << "Value extracted as a ushort: " << usValue << endl;

       mymsg.SeekItem(9);
       mymsg >> uiValue;
       if (BL_convResult != BL_CONV_OK)
         cout << "Warning: failed ushort extraction as a uint.\n";
       else
       {
         cout << "Value extracted as a uint: ";
         cout << uiValue << endl;
       }

       mymsg.SeekItem(9);
       mymsg >> ulValue;
       if (BL_convResult != BL_CONV_OK)
       {
         cout << "Warning: failed ushort extraction as a ulong.\n";
       }
       else
       {
         cout << "Value extracted as a ulong: ";
         cout << ulValue << endl;
       }
     }

     mymsg.SeekItem(10);
     mymsg >> ulValue;
     if (BL_convResult != BL_CONV_OK)
     {
       cout << "(Test 10) Ulong extraction failed:";
       OutFail();
     }
     else
     {
       cout << "(Test 10) Ulong extraction ok:";
       OutOk();
       cout << "Value expected: " << (unsigned long) (ULONG_MAX) << endl;
       cout << "Value extracted as a ulong: " << ulValue << endl;
     }

     // mixed tests

     mymsg.SeekItem(11);
     mymsg >> ucValue;
     if (BL_convResult == BL_CONV_OK)
     {
       cout << "(Test 11) Converting 16 bit value to uchar";
       cout << " should have failed:";
       OutFail();
     }
     else
     {
       cout << "(Test 11) Could not convert 16 bit value to uchar:";
       OutOk();
     }

     mymsg.SeekItem(12);
     mymsg >> ucValue;
     if (BL_convResult == BL_CONV_OK)
     {
       cout << "(Test 12) Success converting 16 bit value to uchar:";
       OutOk();
       cout << "Value expected: " << (short) (UCHAR_MAX) << endl;
       cout << "Value extracted: " << ucValue << endl;
     }
     else
     {
       cout << "(Test 12) Could not convert 16 bit value to uchar:";
       OutFail();
     }

     mymsg.SeekItem(13);
     mymsg >> usValue;
     if (BL_convResult == BL_CONV_OK)
     {
       cout << "(Test 13) Converting 32 bit value to ushort";
       cout << " should have failed:";
       OutFail();
     }
     else
     {
       cout << "(Test 13) Could not convert 32 bit value to ushort:";
       OutOk();
     }

     mymsg.SeekItem(14);
     mymsg >> usValue;
     if (BL_convResult == BL_CONV_OK)
     {
       cout << "(Test 14) Success converting 32 bit value to ushort:";
       OutOk();
       cout << "Value expected: " << (long) (USHRT_MAX) << endl;
       cout << "Value extracted: " << usValue << endl;
     }
     else
     {
       cout << "(Test 14) Could not convert 32 bit value to ushort:";
       OutFail();
     }

     mymsg.SeekItem(15);
     mymsg >> cValue;
     if (BL_convResult == BL_CONV_OK)
     {
       cout << "(Test 15) Converting 8 bit uvalue to char";
       cout << " should have failed:";
       OutFail();
     }
     else
     {
       cout << "(Test 15) Could not convert 8 bit uvalue to char:";
       OutOk();
     }

     mymsg.SeekItem(16);
     mymsg >> cValue;
     if (BL_convResult == BL_CONV_OK)
     {
       cout << "(Test 16) Success converting 16 bit uvalue to char:";
       OutOk();
       cout << "Value expected: " << (unsigned short) (SCHAR_MAX) << endl;
       cout << "Value extracted: " << cValue << endl;
     }
     else
     {
       cout << "(Test 16) Could not convert 16 bit uvalue to char:";
       OutFail();
     }

     mymsg.SeekItem(17);
     mymsg >> sValue;
     if (BL_convResult == BL_CONV_OK)
     {
       cout << "(Test 17) Converting 32 bit uvalue to short";
       cout << " should have failed:";
       OutFail();
     }
     else
     {
       cout << "(Test 17) Could not convert 32 bit uvalue to short:";
       OutOk();
     }

     mymsg.SeekItem(18);
     mymsg >> sValue;
     if (BL_convResult == BL_CONV_OK)
     {
       cout << "(Test 18) Success converting 32 bit uvalue to short:";
       OutOk();
       cout << "Value expected: " << (unsigned long) (SHRT_MAX) << endl;
       cout << "Value extracted: " << sValue << endl;
     }
     else
     {
       cout << "(Test 18) Could not convert 32 bit uvalue to short:";
       OutFail();
     }

     // fp tests

     mymsg.SeekItem(19);
     mymsg >> fValue;
     if (BL_convResult == BL_CONV_OK)
     {
       cout << "(Test 19) Converting double to float should have failed:";
       OutFail();
     }
     else
     {
       cout << "(Test 19) Could not convert double to float:";
       OutOk();
     }

     mymsg.SeekItem(20);
     mymsg >> fValue;
     if (BL_convResult == BL_PREC_LOSS)
     {
       cout << "(Test 20) Success converting double to float:";
       OutOk();
       cout << "Value expected: " << (double) FLT_MAX << endl;
       cout << "Value extracted: " << fValue << endl;
     }
     else
     {
       cout << "(Test 20) Could not convert double to float:";
       OutFail();
     }

     mymsg.SeekItem(21);
     mymsg >> dValue;
     if (BL_convResult == BL_CONV_OK)
     {
       cout << "(Test 21) Success converting float to double:";
       OutOk();
       cout << "Value expected: " << (float) FLT_MAX << endl;
       cout << "Value extracted: " << dValue << endl;
     }
     else
     {
       cout << "(Test 21) Could not convert float to double:";
       OutFail();
     }

     mymsg.SeekItem(22);
     mymsg >> fValue;
     if ((BL_convResult == BL_CONV_OK) || (BL_convResult == BL_PREC_LOSS))
     {
       cout << "(Test 22) Converting double to float should have failed:";
       OutFail();
     }
     else
     {
       cout << "(Test 22) Could not convert double to float:";
       OutOk();
     }

     mymsg.SeekItem(23);
     mymsg >> fValue;
     if ((BL_convResult == BL_CONV_OK) || (BL_convResult == BL_PREC_LOSS))
     {
       cout << "(Test 23) Success converting double to float:";
       OutOk();
       cout << "Value expected: " << (double) FLT_MIN << endl;
       cout << "Value extracted: " << fValue << endl;
     }
     else
     {
       cout << "(Test 23) Could not convert double to float:";
       OutFail();
     }

     mymsg.SeekItem(24);
     mymsg >> dValue;
     if ((BL_convResult == BL_PREC_LOSS) || (BL_convResult == BL_CONV_OK))
     {
       cout << "(Test 24) Success converting float to double:";
       OutOk();
       cout << "Value expected: " << (float) FLT_MIN << endl;
       cout << "Value extracted: " << dValue << endl;
     }
     else
     {
       cout << "(Test 24) Could not convert float to double:";
       OutFail();
     }

     mymsg.SeekItem(25);
     mymsg >> rwValue;
     if (BL_convResult == BL_CONV_OK)
     {
       if (rwValue.ulDataLen != RW_DATA_SIZE)
       {
         cout << "(Test 25) Rawdata object size is not right:";
         OutFail();
       }
       else
       {
         cout << "(Test 25) Rawdata object size matches:";
         OutOk();
       }

       free(rwValue.pcData);
     }

     // fp to int test

     mymsg.SeekItem(26);
     mymsg >> usValue;
     if (BL_convResult == BL_FP_TO_INT)
     {
       cout << "(Test 26) Converting float to ushort should have failed:";
       OutFail();
     }
     else
     {
       cout << "(Test 26) Could not convert float to ushort:";
       OutOk();
       cout << "Value expected: " << (unsigned long) (USHRT_MAX+1ul);
       cout << endl;

       mymsg.SeekItem(26);
       mymsg >> ulValue;
       if (BL_convResult != BL_FP_TO_INT)
       {
         cout << "Warning: failed float extraction as a ulong.\n";
       }
       else
       {
         cout << "Value extracted as a ulong: " << ulValue << endl;
       }
     }

     mymsg.SeekItem(27);
     mymsg >> usValue;
     if (BL_convResult != BL_FP_TO_INT)
     {
       cout << "(Test 27) Could not convert double to ushort:";
       OutFail();
     }
     else
     {
       cout << "(Test 27) Success converting double to ushort:";
       OutOk();
       cout << "Value expected: " << (unsigned short) (USHRT_MAX);
       cout << endl;
       cout << "Value extracted as a ushort: " << usValue << endl;

       mymsg.SeekItem(27);
       mymsg >> ulValue;
       if (BL_convResult != BL_FP_TO_INT)
       {
         cout << "Warning: failed double extraction as a ulong.\n";
       }
       else
       {
         cout << "Value extracted as a ulong: " << ulValue << endl;
       }
     }

     mymsg.SeekItem(28);
     mymsg >> sValue;
     if (BL_convResult == BL_FP_TO_INT)
     {
       cout << "(Test 28) Converting float to short should have failed:";
       OutFail();
     }
     else
     {
       cout << "(Test 28) Could not convert float to short:";
       OutOk();
       cout << "Value expected: " << (long) (SHRT_MIN-1l) << endl;

       mymsg.SeekItem(28);
       mymsg >> lValue;
       if (BL_convResult != BL_FP_TO_INT)
       {
         cout << "Warning: failed float extraction as a long.\n";
       }
       else
       {
         cout << "Value extracted as a long: " << lValue << endl;
       }
     }

     mymsg.SeekItem(29);
     mymsg >> sValue;
     if (BL_convResult != BL_FP_TO_INT)
     {
       cout << "(Test 29) Could not convert double to short:";
       OutFail();
     }
     else
     {
       cout << "(Test 29) Success converting double to short:";
       OutOk();
       cout << "Value expected: " << (short) (SHRT_MIN) << endl;
       cout << "Value extracted as a short: " << sValue << endl;

       mymsg.SeekItem(29);
       mymsg >> lValue;
       if (BL_convResult != BL_FP_TO_INT)
       {
         cout << "Warning: failed double extraction as a long.\n";
       }
       else
       {
         cout << "Value extracted as a long: " << lValue << endl;
       }
     }

     // int to fp test

     mymsg >> fValue;
     if (BL_convResult != BL_INT_TO_FP)
     {
       cout << "(Test 30) Could not convert ulong to float:";
       OutFail();
     }
     else
     {
       cout << "(Test 30) Success converting ulong to float:";
       OutOk();
       cout << "Value expected: " << (float) (USHRT_MAX) << endl;
       cout << "Value extracted as a float: " << fValue << endl;

       mymsg.SeekItem(30);
       mymsg >> dValue;
       if (BL_convResult != BL_INT_TO_FP)
       {
         cout << "Warning: failed ulong extraction as a double.\n";
       }
       else
       {
         cout << "Value extracted as a double: " << dValue << endl;
       }
     }

     break;

    case 2: case 4:
     do
     {
       cout << "Message item:\n";
       cout << "1  - char\n";
       cout << "2  - unsigned char\n";
       cout << "3  - short\n";
       cout << "4  - unsigned short\n";
       cout << "5  - int\n";
       cout << "6  - unsigned int\n";
       cout << "7  - long\n";
       cout << "8  - unsigned long\n";
       cout << "9  - float\n";
       cout << "10 - double\n";
       cout << "11 - int 16\n";
       cout << "12 - unsigned int 16\n";
       cout << "13 - int 32\n";
       cout << "14 - unsigned int 32\n";
       cout << "15 - int 64\n";
       cout << "16 - unsigned int 64\n";
       cout << "17 - string\n";
       cout << "18 - string as rawdata\n";
       cout << "19 - SEND MESSAGE\n\n";

       cout << "Make your choice: ";
       cin >> sChoice;

       if (sChoice != 19)
       {
         cout << "Input value: ";
         switch (sChoice)
         {
           case 1 :
            cin >> cValue;
            mymsg << cValue;
            break;

           case 2 :
            cin >> ucValue;
            mymsg << ucValue;
            break;

           case 3 :
            cin >> sValue;
            mymsg << sValue;
            break;

           case 4 :
            cin >> usValue;
            mymsg << usValue;
            break;

           case 5 :
            cin >> iValue;
            mymsg << iValue;
            break;

           case 6 :
            cin >> uiValue;
            mymsg << uiValue;
            break;

           case 7 :
            cin >> lValue;
            mymsg << lValue;
            break;

           case 8 :
            cin >> ulValue;
            mymsg << ulValue;
            break;

           case 9 :
            cin >> fValue;
            mymsg << fValue;
            break;

           case 10:
            cin >> dValue;
            mymsg << dValue;
            break;

           case 11:
            cin >> lValue;
            mymsg << BL_ToInt16(lValue);
            break;

           case 12:
            cin >> lValue;
            mymsg << BL_ToUint16(lValue);
            break;

           case 13:
            cin >> lValue;
            mymsg << BL_ToInt32(lValue);
            break;

           case 14:
            cin >> lValue;
            mymsg << BL_ToUint32(lValue);
            break;

           case 15:
            cin >> lValue;
            mymsg << BL_ToInt64(lValue);
            break;

           case 16:
            cin >> lValue;
            mymsg << BL_ToUint64(lValue);
            break;

           case 17:
            cpValue = (char *) malloc(MAX_STRING);
            cin >> cpValue;
            mymsg << cpValue;
            free(cpValue);
            break;

           case 18:
            cpValue = (char *) malloc(MAX_STRING);
            cin >> cpValue;
            mymsg << BL_ToRawData((char *) cpValue, strlen(cpValue)+1);
            free(cpValue);
            break;

         }

         // Linux stream library hate repositioning
         #ifndef BL_MSD_INCLUDE

           #ifndef BL_LITTLE_END

             cin.seekg(0l,ios::end);

           #endif

         #endif

       }
     } while (sChoice != 19);

     cout << "\nInsert the message type tag: ";
     cin >> i32UserType;

     mymsg.SetType(i32UserType);

     if (!mymsg.MsgSave("test.dat"))
     {
       cout << "Error saving to test.dat";
       exit(1);
     }

     cout << "\n============== Message sent ==============";
     cout << endl << endl;

     if (sTestType == 2)
       break;

     mymsg.Clear();

    case 3:
     if (!mymsg.MsgLoad("test.dat"))
     {
       cout << "Error saving to test.dat";
       exit(1);
     }

     cout << "============ Message received ============";
     cout << endl << endl;

     cout << "Message type    : " << mymsg.GetType()    << endl;
     cout << "Message length  : " << mymsg.GetDataLen() << endl;

     cout << "Message contents:" << endl;
     sGap = -2;

     while (1)
     {
       cout << "\nCurrent item: " << mymsg.SkipItem(0) << endl;
       if ( mymsg.SkipItem(0) == 1 )
         cout << "Begin of message\n";
       switch (mymsg.GetItemType())
       {
         case BL_CHAR:
          mymsg >> cValue;
          cout << "char:     " << cValue;
          break;

         case BL_UCHAR:
          mymsg >> ucValue;
          cout << "uchar:    " << ucValue;
          break;

         case BL_I16: case BL_I32: case BL_I64:
          mymsg >> lValue;
          cout << "integer:  " << lValue;
          break;

         case BL_UI16: case BL_UI32: case BL_UI64:
          mymsg >> ulValue;
          cout << "uinteger: " << ulValue;
          break;

         case BL_FLOAT:
          mymsg >> fValue;
          cout << "float:    " << fValue;
          break;

         case BL_DOUBLE:
          mymsg >> dValue;
          cout << "double:   " << dValue;
          break;

         case BL_CHARP:
          mymsg >> cpValue;
          cout << "string:   " << cpValue;
          free(cpValue);
          break;

         case BL_RAWDATA:
          mymsg >> rwValue;
          cout << "raw data: " << rwValue.pcData;
          free(rwValue.pcData);
          break;

         case BL_NOT_VALID:
          cout << "End of message";
          sGap = -1;
          break;
       }

       cout << "\nType > for next item, < for previous item,\n";
       cout << "* to delete item, ! to end program: ";

       while ((strchr("<>!*", (cValue = getchar())) == NULL)
              || (cValue == 0));

       switch (cValue)
       {
         case '>':
           break;
         case '<':
           mymsg.SkipItem(sGap);
           sGap = -2;
           break;
         case '*':
           if ((!mymsg.EOMsg()) || (sGap==-2))
           {
             mymsg.SkipItem(-1);
             ulValue = mymsg.SkipItem(0);
             mymsg.DeleteItem(ulValue);
             mymsg.SeekItem(ulValue);
           }
           else
             if ((mymsg.SkipItem(0) == 2) && (sGap==-2))
               mymsg.DeleteItem(1);
           break;
       }

       if (cValue=='!')
         break;

     }
     break;
  }

  return 0;

}

void OutOk()
{
 cout << " test passed.\n";
 return;
}

void OutFail()
{
 cout << " test NOT passed.\n";
 return;
}

//******************************************************************
//************************** END OF FILE ***************************
//******************************************************************
